@@ -74,6 +74,7 @@ public class Restore extends BasicCommandlineOptions {
 
     CLIOption filenameOption = null;
     CLIOption dryRun = null;
+    CLIOption configDbNameOption = null;
 
     public static void main(final String[] args) {
         final Restore restore = new Restore();
@@ -84,9 +85,11 @@ public class Restore extends BasicCommandlineOptions {
         setDefaultCommandLineOptions(parser);
         filenameOption = setShortLongOpt(parser, 'f', "filename","Comma-separated list of filenames with full path", true, NeededQuadState.needed);
         dryRun = setShortLongOpt(parser, 'n', "dry-run","Activate this option if do not want to apply the changes to the database", false, NeededQuadState.needed);
+        configDbNameOption = setShortLongOpt(parser, 'd', "configdb", "(Optional) The name of the ConfigDB schema. If not set, ConfigDB name is determined by \"writeUrl\" property in file configdb.properties", true, NeededQuadState.notneeded);
     }
 
     public Restore() {
+        super();
     }
 
     public void start(final String[] args) {
@@ -105,14 +108,15 @@ public class Restore extends BasicCommandlineOptions {
             // get rmi ref
             final OXContextRestoreInterface oxrestore = getContextRestoreInterface();
 
+            final String configDbName = (String) parser.getOptionValue(configDbNameOption);
 
             final String filenames = (String) parser.getOptionValue(filenameOption);
 
             final String[] filenamearray = filenames.split(",");
             if (null == parser.getOptionValue(dryRun)) {
-                System.out.println(oxrestore.restore(ctx, filenamearray, auth, false));
+                System.out.println(oxrestore.restore(ctx, filenamearray, isEmpty(configDbName) ? null : configDbName, auth, false));
             } else {
-                System.out.println(oxrestore.restore(ctx, filenamearray, auth, true));
+                System.out.println(oxrestore.restore(ctx, filenamearray, isEmpty(configDbName) ? null : configDbName, auth, true));
             }
 
             sysexit(0);
@@ -163,4 +167,35 @@ public class Restore extends BasicCommandlineOptions {
         return (OXContextRestoreInterface) Naming.lookup(RMI_HOSTNAME + OXContextRestoreInterface.RMI_NAME);
     }
 
+    private static boolean isEmpty(final String string) {
+        if (null == string) {
+            return true;
+        }
+        final int len = string.length();
+        boolean isWhitespace = true;
+        for (int i = 0; isWhitespace && i < len; i++) {
+            switch (string.charAt(i)) {
+            case 9: // 'unicode: 0009
+            case 10: // 'unicode: 000A'
+            case 11: // 'unicode: 000B'
+            case 12: // 'unicode: 000C'
+            case 13: // 'unicode: 000D'
+            case 28: // 'unicode: 001C'
+            case 29: // 'unicode: 001D'
+            case 30: // 'unicode: 001E'
+            case 31: // 'unicode: 001F'
+            case ' ': // Space
+                // case Character.SPACE_SEPARATOR:
+                // case Character.LINE_SEPARATOR:
+            case Character.PARAGRAPH_SEPARATOR:
+                isWhitespace = true;
+                break;
+            default:
+                isWhitespace = false;
+                break;
+            }
+        }
+        return isWhitespace;
+    }
+
 }
